unit Dialogs;

{ Pascal Interface to the Macintosh Libraries }

{ Copyright  Apple Computer Inc. }
{ All Rights Reserved }

{ Adapted for use with THINK Pascal 4.0.x by Marco Piovanelli }

interface
	uses
		Types, Windows;

	const

{ new, more standard names for dialog item constants }
		kControlDialogItem = ctrlItem;
		kButtonDialogItem = ctrlItem + btnCtrl;
		kCheckBoxDialogITem = ctrlItem + chkCtrl;
		kRadioButtonDialogItem = ctrlItem + radCtrl;
		kResourceControlDialogItem = ctrlItem + resCtrl;
		kStaticTextDialogItem = statText;
		kEditTextDialogItem = editText;
		kIconDialogItem = iconItem;
		kPictureDialogItem = picItem;
		kUserDialogItem = userItem;
		kItemDisableBit = itemDisable;
		kStdOkItemIndex = ok;
		kStdCancelItemIndex = cancel;
		kStopIcon = stopIcon;
		kNoteIcon = noteIcon;
		kCautionIcon = cautionIcon;

	type
		DITLMethod = INTEGER;


	const
		overlayDITL = 0;
		appendDITLRight = 1;
		appendDITLBottom = 2;


	type

		DialogPropertyTag = OSType;

		DialogRef = DialogPtr;

		SoundProcPtr = ProcPtr;  { PROCEDURE Sound(soundNumber: INTEGER); }
		ModalFilterProcPtr = ProcPtr;  { FUNCTION ModalFilter(theDialog: DialogPtr; VAR theEvent: EventRecord; VAR itemHit: INTEGER): BOOLEAN; }
		UserItemProcPtr = ProcPtr;  { PROCEDURE UserItem(theWindow: WindowPtr; itemNo: INTEGER); }
		SoundUPP = UniversalProcPtr;
		ModalFilterUPP = UniversalProcPtr;
		UserItemUPP = UniversalProcPtr;

	const
		uppSoundProcInfo = $00000080; { PROCEDURE (2 byte param); }
		uppModalFilterProcInfo = $00000FD0; { FUNCTION (4 byte param, 4 byte param, 4 byte param): 1 byte result; }
		uppUserItemProcInfo = $000002C0; { PROCEDURE (4 byte param, 2 byte param); }

	function NewSoundProc (userRoutine: SoundProcPtr): SoundUPP;
	inline
		$2E9F;

	function NewModalFilterProc (userRoutine: ModalFilterProcPtr): ModalFilterUPP;
	inline
		$2E9F;

	function NewUserItemProc (userRoutine: UserItemProcPtr): UserItemUPP;
	inline
		$2E9F;

	procedure CallSoundProc (soundNumber: INTEGER;
									userRoutine: SoundUPP);
	inline
		$205F, $4E90;

	function CallModalFilterProc (theDialog: DialogPtr;
									var theEvent: EventRecord;
									var itemHit: INTEGER;
									userRoutine: ModalFilterUPP): BOOLEAN;
	inline
		$205F, $4E90;

	procedure CallUserItemProc (theWindow: WindowPtr;
									itemNo: INTEGER;
									userRoutine: UserItemUPP);
	inline
		$205F, $4E90;

	type

		SelectionRange = record
				beginSelection: SInt16;
				endSelection: SInt16;
			end;

	procedure GetDialogItem (theDialog: DialogRef;
									itemNo: INTEGER;
									var itemType: INTEGER;
									var item: Handle;
									var box: Rect);
	inline
		$A98D;

	procedure SetDialogItem (theDialog: DialogRef;
									itemNo: INTEGER;
									itemType: INTEGER;
									item: Handle; {CONST}
									var box: Rect);
	inline
		$A98E;

	procedure HideDialogItem (theDialog: DialogRef;
									itemNo: INTEGER);
	inline
		$A827;

	procedure ShowDialogItem (theDialog: DialogRef;
									itemNo: INTEGER);
	inline
		$A828;

	procedure SelectDialogItemText (theDialog: DialogRef;
									itemNo: INTEGER;
									strtSel: INTEGER;
									endSel: INTEGER);
	inline
		$A97E;

	procedure GetDialogItemText (item: Handle;
									var text: Str255);
	inline
		$A990;

	procedure SetDialogItemText (item: Handle;
									text: ConstStr255Param);
	inline
		$A98F;

	function FindDialogItem (theDialog: DialogRef;
									thePt: Point): INTEGER;
	inline
		$A984;

	function NewColorDialog (dStorage: univ Ptr; {CONST}
									var boundsRect: Rect;
									title: ConstStr255Param;
									visible: BOOLEAN;
									procID: INTEGER;
									behind: WindowRef;
									goAwayFlag: BOOLEAN;
									refCon: LONGINT;
									items: Handle): DialogRef;
	inline
		$AA4B;

	function GetAlertStage: INTEGER;
	inline
		$3EB8, $0A9A;			{ move.w ACount, (sp) }

	procedure ResetAlertStage;
	inline
		$4278, $0A9A;			{ clr.w ACount }

	procedure DialogCut (theDialog: DialogRef);
	procedure DialogPaste (theDialog: DialogRef);
	procedure DialogCopy (theDialog: DialogRef);
	procedure DialogDelete (theDialog: DialogRef);
	procedure SetDialogFont (value: INTEGER);
	inline
		$31DF, $0AFA;			{ move.w (sp)+, DlgFont }

	function CountDITL (theDialog: DialogRef): Integer;
	inline
		$3F3C, $0403, $204F, $A08B, $5C4F, $3E80;

	procedure AppendDITL (theDialog: DialogRef;
									theDITL: Handle;
									method: DITLMethod);
	inline
		$3F3C, $0402, $204F, $A08B, $4FEF, $000C;

	procedure ShortenDITL (theDialog: DialogRef;
									numberItems: Integer);
	inline
		$3F3C, $0404, $204F, $A08B, $504F;

	function GetStdFilterProc (var theProc: ModalFilterUPP): OSErr;
	inline
		$303C, $0203, $AA68;

	function SetDialogDefaultItem (theDialog: DialogRef;
									newItem: INTEGER): OSErr;
	inline
		$303C, $0304, $AA68;

	function SetDialogCancelItem (theDialog: DialogRef;
									newItem: INTEGER): OSErr;
	inline
		$303C, $0305, $AA68;

	function SetDialogTracksCursor (theDialog: DialogRef;
									tracks: BOOLEAN): OSErr;
	inline
		$303C, $0306, $AA68;

implementation
end.